#!/bin/bash
#
# Copyright (C) 2017 CompuLab, Ltd.
# Author: Nikita Kiryanov <nikita@compulab.co.il>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

usage()
{
	cat >&2 <<EOF
Usage: $0 <spl image path> <u-boot image path> [-o <path to output file>]

DESCRIPTION
	This script combines the SPL and U-Boot binaries into a single
	cl-som-imx7-firmware image that can be written into on-board SPI flash or SD
	card.

	The spl binary is placed at 1K offset in the cl-som-imx7-firmware image,
	while u-boot is placed in a 64K offset in the cl-som-imx7-firmware image.
	The resulting image should be written in offset 0 of the chosen storage.
	
OPTIONS
	-o	Normally the script's output is a file called "cl-som-imx7-firmware",
		located in the script's directory. Use this option to provide a
		different output location and filename.
EOF
}

OUTPUT_FILE=cl-som-imx7-firmware

spl_path="$1"
uboot_path="$2"
if [[ -z "$spl_path" || -z "$uboot_path" ]] ; then
	usage
	exit 1
fi

shift
shift

if [ "$#" -ge  1 ]; then
	case "$1" in
	-o)
		shift
		OUTPUT_FILE="$1"
		break
		;;
	*)
		echo "Unknown option $1"
		usage
		exit 1
		;;
	esac
fi

echo -e "\nGenerating firmware image from the following binaries:"
echo -e ">>> SPL: `grep -oazE "SPL [0-9]+\.[0-9]+.* \(... +[0-9]+ [0-9]+ - [0-9]+:[0-9]+:[0-9]+\)" $spl_path`"
echo -e ">>> U-Boot: `grep -oazE "U-Boot [0-9]+\.[0-9]+.* \(... +[0-9]+ [0-9]+ - [0-9]+:[0-9]+:[0-9]+ \+[0-9]+\)" $uboot_path`\n"
read -p "Press any key to continue or Ctrl-C to abort"

dd if=/dev/zero count=640 bs=1K | tr '\000' '\377' > "$OUTPUT_FILE"
dd if="$spl_path" of="$OUTPUT_FILE" bs=1K seek=1 conv=notrunc
dd if="$uboot_path" of="$OUTPUT_FILE" bs=1K seek=64 conv=notrunc

